import Schema from 'async-validator';
import _clone from 'lodash/clone';
import _setWith from 'lodash/setWith';
import _get from 'lodash/get';
import { __assign, __rest } from 'tslib';
import React__default, { useReducer, useCallback, useMemo, useContext, useEffect, createContext } from 'react';
import classNames from 'classnames';

function getRules(prop, rules, trigger) {
    if (!rules || !rules[prop]) {
        return [];
    }
    var result = Array.isArray(rules[prop]) ? rules[prop] : [rules[prop]];
    if (!trigger) {
        return result;
    }
    return result.filter(function (rule) {
        return !rule.trigger || rule.trigger.indexOf(trigger) !== -1;
    });
}
function isRequired(prop, rules) {
    var data = getRules(prop, rules);
    if (data.length) {
        for (var index = 0; index < data.length; index++) {
            var rule = data[index];
            if (rule.required) {
                return true;
            }
        }
    }
    return false;
}
function getValidKeys(formValue, keys) {
    var allKeys = Object.keys(formValue);
    if (!keys) {
        return allKeys;
    }
    if (!Array.isArray(keys)) {
        keys = [keys];
    }
    return keys.filter(function (v) { return allKeys.indexOf(v) !== -1; });
}
function find(arr, fn) {
    for (var _i = 0, arr_1 = arr; _i < arr_1.length; _i++) {
        var item = arr_1[_i];
        if (fn(item)) {
            return item;
        }
    }
    return undefined;
}

function baseValidate(formValue, keys, rules, trigger) {
    var descriptor = {};
    var data = {};
    var validKeys = getValidKeys(formValue, keys);
    validKeys.forEach(function (key) {
        var rule = getRules(key, rules, trigger);
        var value = _get(formValue, key);
        data[key] = value;
        if (rule.length) {
            descriptor[key] = rule;
        }
    });
    if (Object.keys(descriptor).length === 0) {
        return Promise.resolve(data);
    }
    return new Promise(function (resolve, reject) {
        var validator = new Schema(descriptor);
        validator.validate(data, function (errors) {
            if (errors) {
                var error_1 = {};
                validKeys.forEach(function (key) {
                    error_1[key] = find(errors, function (o) { return o.field === key; });
                });
                reject(error_1);
            }
            else {
                resolve(data);
            }
        });
    });
}
function init(initialValue) {
    return { value: initialValue, error: {} };
}
function reducer(state, action) {
    var value = state.value, error = state.error;
    var type = action.type, payload = action.payload;
    switch (type) {
        case 'setValue':
            value = _clone(value);
            Object.keys(payload).forEach(function (key) {
                _setWith(value, key, payload[key], _clone);
            });
            return { value: value, error: error };
        case 'setError':
            error = _clone(error);
            Object.keys(payload).forEach(function (key) {
                _setWith(error, key, payload[key], _clone);
            });
            return { value: value, error: error };
        case 'reset':
            return init(payload);
        default:
            throw new Error();
    }
}
function useFormx(initialValue, rules) {
    if (initialValue === void 0) { initialValue = {}; }
    var _a = useReducer(reducer, initialValue, init), formState = _a[0], dispatch = _a[1];
    var setFieldsValue = useCallback(function (payload) {
        dispatch({ type: 'setValue', payload: payload });
    }, []);
    var setFieldsError = useCallback(function (payload) {
        dispatch({ type: 'setError', payload: payload });
    }, []);
    var dispatchField = useCallback(function (payload) {
        var _a, _b;
        var type = payload.type, prop = payload.prop, value = payload.value, error = payload.error;
        if (type === 'unmount') {
            setFieldsValue((_a = {}, _a[prop] = undefined, _a));
            return;
        }
        var data = (_b = {}, _b[prop] = value, _b);
        if (type === 'change') {
            setFieldsValue(data);
        }
        baseValidate(data, prop, rules, type)
            .then(function () {
            var _a;
            if (error) {
                setFieldsError((_a = {}, _a[prop] = undefined, _a));
            }
        })
            .catch(function (err) {
            setFieldsError(err);
        });
    }, [rules]);
    var resetFields = function () {
        dispatch({ type: 'reset', payload: initialValue });
    };
    var getField = function (prop) {
        return [_get(formState.value, prop), _get(formState.error, prop)];
    };
    var validate = function (keys) {
        return baseValidate(formState.value, keys, rules).catch(function (err) {
            setFieldsError(err);
            throw err;
        });
    };
    var bindFormx = function (prop) {
        var required = isRequired(prop, rules);
        var _a = getField(prop), value = _a[0], error = _a[1];
        return {
            required: required,
            value: value,
            error: error,
            prop: prop,
            key: prop,
            dispatch: dispatchField
        };
    };
    return __assign({}, formState, { bindFormx: bindFormx,
        validate: validate,
        setFieldsValue: setFieldsValue,
        setFieldsError: setFieldsError,
        resetFields: resetFields,
        getField: getField });
}

var FormxContext = createContext({});

var Provider = FormxContext.Provider;
function Formx(props) {
    var _a = props.labelPosition, labelPosition = _a === void 0 ? 'right' : _a, _b = props.labelSuffix, labelSuffix = _b === void 0 ? ':' : _b, labelWidth = props.labelWidth, disabled = props.disabled, className = props.className, restProps = __rest(props, ["labelPosition", "labelSuffix", "labelWidth", "disabled", "className"]);
    var memoized = useMemo(function () {
        return { labelPosition: labelPosition, labelWidth: labelWidth, labelSuffix: labelSuffix, disabled: disabled };
    }, [labelPosition, labelWidth, labelSuffix, disabled]);
    var formClasses = classNames('easy-formx', "easy-formx--label-" + labelPosition, className);
    return (React__default.createElement(Provider, { value: memoized },
        React__default.createElement("form", __assign({ className: formClasses }, restProps))));
}

function isObject(obj) {
    return obj !== null && typeof obj === 'object';
}
function getValue(event) {
    if (!isObject(event) || !event.target) {
        return event;
    }
    var target = event.target;
    var value = target.type === 'checkbox' ? target.checked : target.value;
    return value;
}
function getStyle(labelPosition, labelWidth) {
    var label = {};
    var content = {};
    if (labelPosition === 'left' || labelPosition === 'right') {
        label.textAlign = labelPosition;
        if (labelWidth) {
            label.width = labelWidth;
            content.marginLeft = labelWidth;
        }
    }
    return { label: label, content: content };
}
function FormxItem(props) {
    var prop = props.prop, value = props.value, error = props.error, required = props.required, dispatch = props.dispatch, 
    // above from `bindFormx
    label = props.label, labelStyle = props.labelStyle, _a = props.trigger, trigger = _a === void 0 ? 'onChange' : _a, _b = props.valuePropName, valuePropName = _b === void 0 ? 'value' : _b, children = props.children, rest = __rest(props, ["prop", "value", "error", "required", "dispatch", "label", "labelStyle", "trigger", "valuePropName", "children"]);
    var _c = useContext(FormxContext), labelPosition = _c.labelPosition, labelSuffix = _c.labelSuffix, labelWidth = _c.labelWidth, disabled = _c.disabled;
    useEffect(function () {
        return function () {
            if (prop && dispatch) {
                dispatch({ type: 'unmount', prop: prop, value: value });
            }
        };
    }, []);
    var message = error && error.message;
    var handleChange = function (event) {
        var value = getValue(event);
        if (prop && dispatch) {
            dispatch({ type: 'change', prop: prop, value: value, error: error });
        }
    };
    var handleBlur = function () {
        if (prop && dispatch) {
            dispatch({ type: 'blur', prop: prop, value: value, error: error });
        }
    };
    var itemClasses = classNames('easy-formx-item', {
        'has-error': error,
        'easy-formx-item--with-help': message
    });
    var labelClasses = classNames('easy-formx-item__label', {
        'is-required': required
    });
    var style = getStyle(labelPosition, labelWidth);
    var items = children;
    if (prop) {
        items = React__default.Children.map(children, function (child, index) {
            var _a;
            if (index === 0 && React__default.isValidElement(child)) {
                return React__default.cloneElement(child, (_a = {
                        disabled: child.props.disabled || disabled
                    },
                    _a[valuePropName] = value,
                    _a[trigger] = handleChange,
                    _a));
            }
            return child;
        });
    }
    return (React__default.createElement("div", __assign({}, rest, { className: itemClasses, onBlur: handleBlur }),
        label && (React__default.createElement("label", { className: labelClasses, style: __assign({}, style.label, labelStyle) },
            label,
            React__default.createElement("span", { className: "esay-formx-item__suffix" }, labelSuffix))),
        React__default.createElement("div", { className: "easy-formx-item__content", style: style.content },
            items,
            message && React__default.createElement("div", { className: "easy-formx-item__error" }, message))));
}

export { useFormx, Formx, FormxItem };
//# sourceMappingURL=index.es.js.map
